/*! 
  @file rx_jpeg.h
	
  @brief JPEGt@Cǂݍ݁Co
	- libjpeggp
	- http://www.ijg.org/
 
  @author Makoto Fujisawa
  @date 2011-06
*/
// FILE --rx_jpeg.h--

#ifndef _RX_JPEG_H_
#define _RX_JPEG_H_


//-----------------------------------------------------------------------------
// CN[ht@C
//-----------------------------------------------------------------------------
#include <cstdio>

#include <vector>
#include <string>

#include <jpeglib.h>
#include <jerror.h>


//-----------------------------------------------------------------------------
// JPEGt@C̓ǂݍ݂Ə
//-----------------------------------------------------------------------------
/*!
 * JPEGt@C̓ǂݍ
 * @param[in] fn t@C
 * @param[out] w,h 摜TCY
 * @param[out] c 摜̐F[x
 * @return WJς݉摜f[^
 */
static unsigned char* ReadJpegFile(const std::string &fn, int &w, int &h, int &c)
{
	jpeg_decompress_struct cinfo;
	jpeg_error_mgr jerr;

	// JPEG𓀗pIuWFNg
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

	// t@CoCi[hŊJ
	FILE *fp;
	if((fp = fopen(fn.c_str(), "rb")) == NULL){
		fprintf(stderr, "jpeg error : cannot open %s file\n", fn.c_str());
		return 0;
	}

	// 𓀂f[^w
	jpeg_stdio_src(&cinfo, fp);

	// t@Cwb_̓ǂݍ
	jpeg_read_header(&cinfo, TRUE);

	// 摜F[x
	c = cinfo.num_components;
	if(!c){
		fprintf(stderr, "jpeg error : the number of color components is zero\n");
		return 0;
	}

	// 摜TCY
	w = cinfo.image_width;
	h = cinfo.image_height;
	if(!w || !h){
		fprintf(stderr, "jpeg error : size of the image is zero\n");
		return 0;
	}

	// f[^
	jpeg_start_decompress(&cinfo);

	// f[^
	JSAMPARRAY buf;
	buf = (*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo, JPOOL_IMAGE, w*c, (JDIMENSION)1);

	// o̓f[^
	unsigned char* img = new unsigned char[w*h*c];

	// 1sRs[
	unsigned char* dst_ptr = img;
	unsigned char* src_ptr;
	while(cinfo.output_scanline < (unsigned int)h){
		jpeg_read_scanlines(&cinfo, buf, 1);
		src_ptr = buf[0];
		for(int i = 0; i < w*c; ++i){
			*dst_ptr++ = *src_ptr++;
		}
	}
	
	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);
	fclose(fp);

	return img;
}

/*!
 * JPEGt@C̏
 * @param[in] fn t@C
 * @param[in] img 摜f[^
 * @param[in] w,h 摜TCY
 * @param[in] c 摜̐F[x
 * @param[in] quality ki[0,100]
 */
static int WriteJpegFile(const std::string &fn, unsigned char *img, int w, int h, int c, int quality)
{
	jpeg_compress_struct cinfo;
	jpeg_error_mgr jerr;

	// JPEGkpIuWFNg
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_compress(&cinfo);

	// t@CoCi[hŊJ
	FILE *fp;
	if((fp = fopen(fn.c_str(), "wb")) == NULL){
		fprintf(stderr, "jpeg error : cannot open %s file\n", fn.c_str());
		return 0;
	}

	// o̓t@Cw
	jpeg_stdio_dest(&cinfo, fp);

	// 摜t@C
	cinfo.image_width = w;
	cinfo.image_height = h;
	cinfo.input_components = c;
	cinfo.in_color_space = (c == 3 ? JCS_RGB : JCS_GRAYSCALE);

	// kݒ
	jpeg_set_defaults(&cinfo);	// ftHg̃p[^Zbg
	jpeg_set_quality(&cinfo, quality, TRUE);	// 摜iݒ

	// f[^k
	jpeg_start_compress(&cinfo, TRUE);

	// 1so
	unsigned char* src_ptr = img;
	JSAMPARRAY dst_ptr = new JSAMPROW[w*c];
	while(cinfo.next_scanline < (unsigned int)h){
		jpeg_write_scanlines(&cinfo, &src_ptr, 1);
		src_ptr += w*c;
	}
	
	jpeg_finish_compress(&cinfo);
	jpeg_destroy_compress(&cinfo);
	fclose(fp);

	return 1;
}




#endif // #ifdef _RX_JPEG_H_
